@@ -3,14 +3,13 @@
 #include <stdio.h>
 /* For INT_MAX/INT_MIN */
 #include <limits.h>
+/* For malloc. */
+#include <stdlib.h>
 #include "config.h"
 #include "text-fuzzy.h"
 #include "edit-distance-char-trans.h"
-#line 11 "edit-distance.c.tmpl"
+#line 13 "edit-distance.c.tmpl"
 
-/* For malloc. */
-
-#include <stdlib.h>
 
 /* Our unsorted dictionary linked list.       */
 
@@ -87,12 +86,12 @@ int distance_char_trans (
                     text_fuzzy_t * tf)
 
 {
-#line 91 "edit-distance.c.tmpl"
+#line 90 "edit-distance.c.tmpl"
 
 
 
 
-#line 102 "edit-distance.c.tmpl"
+#line 101 "edit-distance.c.tmpl"
     const unsigned char * word1 = (const unsigned char *) tf->b.text;
     int len1 = tf->b.length;
     const unsigned char * word2 = (const unsigned char *) tf->text.text;
@@ -104,8 +103,20 @@ int distance_char_trans (
     item *head = NULL;
 
     unsigned int swapScore,targetCharCount,i;
-    unsigned int matrix[len1 + 2][len2 + 2];
     unsigned int score_ceil = len1 + len2;
+#ifdef __GNUC__
+    unsigned int matrix[len1 + 2][len2 + 2];
+#else
+    unsigned int ** matrix;
+    int d;
+#endif
+
+#ifndef __GNUC__
+    matrix = calloc (len1 + 2, sizeof (unsigned int *));
+    for (i = 0; i < len1 + 2; i++) {
+	matrix[i] = calloc (len2 + 2, sizeof (unsigned int));
+    }
+#endif
 
     if (len1 == 0) {
 	return len2;
@@ -160,8 +171,23 @@ int distance_char_trans (
 
     dict_free (head);
 
+#ifdef __GNUC__
+
     return matrix[len1 + 1][len2 + 1];
 
-#line 306 "edit-distance.c.tmpl"
+#else
+
+    d = matrix[len1 + 1][len2 + 1];
+
+    for (i = 0; i < len1 + 2; i++) {
+	free (matrix[i]);
+    }
+    free (matrix);
+
+    return d;
+
+#endif
+
+#line 359 "edit-distance.c.tmpl"
 }
 
