@@ -1,6 +1,7 @@
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
+#include "config.h"
 #include "text-fuzzy.h"
 #include "edit-distance-char-trans.h"
 #include "edit-distance-int-trans.h"
@@ -73,6 +74,8 @@ error_handler_t text_fuzzy_error_handler =
 
 /* Fail a test, with message. */
 
+#ifdef __GNUC__
+
 #define FAIL_MSG(condition, status, msg, args...)                       \
     if (condition) {                                                    \
         LINE_ERROR (condition, status);                                 \
@@ -84,6 +87,20 @@ error_handler_t text_fuzzy_error_handler =
         return text_fuzzy_status_ ## status;                      \
     }
 
+#else /* __GNUC__ */
+
+#define FAIL_MSG fail_msg
+
+static inline void fail_msg (int condition, int status, char * msg, ...)
+{
+    if (condition) {
+	fprintf (stderr, "%d:%s", status, msg);
+	exit (1);
+    }
+}
+
+#endif /* __GNUC__ */
+
 #define OK return text_fuzzy_status_ok;
 
 /* Call a function and print an error message and return if the
@@ -149,6 +166,7 @@ End:
 
 
 
+
 /* All of the following are automatically generated from
    "edit-distance.h.tmpl" by "make-edit-distance-c.pl". */
 
@@ -162,11 +180,6 @@ End:
 
 #ifdef HEADER
 
-/* If tf->max_distance is set to this value, no maximum distance is
-   used. */
-
-#define NO_MAX_DISTANCE -1
-
 /* Alphabet over unicode characters. */
 
 typedef struct ualphabet {
@@ -234,6 +247,10 @@ typedef struct text_fuzzy {
     /* ASCII alphabet */
     int alphabet[0x100];
 
+    /* The number of characters which were rejected using the ASCII
+       alphabet. */
+    int alphabet_rejections;
+
     /* Unicode alphabet. */
     ualphabet_t ualphabet;
 
@@ -249,6 +266,9 @@ typedef struct text_fuzzy {
        difference is bigger than the maximum edit distance. */
     int length_rejections;
 
+    /* A character which is not in use. */
+    unsigned char invalid_char;
+
     /* Does the user want to use an alphabet filter? Default is yes,
        so this must be set to a non-zero value to switch off use. */
     unsigned int user_no_alphabet : 1;
@@ -281,10 +301,6 @@ text_fuzzy_t;
 
 #endif /* HEADER */
 
-/* The maximum feasible size of the Unicode alphabet. */
-
-#define UALPHABET_MAX_SIZE 0x10000
-
 /* The following calculations need to be done twice, first when
    creating the alphabet and second when looking up a new character in
    it. The macro saves us from exasperating bugs. */
@@ -597,6 +613,7 @@ FUNC (compare_single) (text_fuzzy_t * tf)
 				   are within the maximum edit distance of
 				   each other. */
 
+				tf->alphabet_rejections++;
 				OK;
 			    }
 			}
@@ -672,6 +689,15 @@ FUNC (generate_alphabet) (text_fuzzy_t * text_fuzzy)
     if (unique_characters > max_unique_characters) {
         text_fuzzy->use_alphabet = 0;
     }
+    /* Find an unused slot. This is for the case where the string to
+       match is not in Unicode, but the string which it is matched
+       against is in Unicode. */
+    for (i = 1; i < 0x100; i++) {
+	if (text_fuzzy->alphabet[i] == 0) {
+	    text_fuzzy->invalid_char = i;
+	    break;
+	}
+    }
     OK;
 }
 
@@ -695,6 +721,7 @@ FUNC (begin_scanning) (text_fuzzy_t * text_fuzzy)
 
     text_fuzzy->distance = -1;
     text_fuzzy->ualphabet.rejections = 0;
+    text_fuzzy->alphabet_rejections = 0;
     text_fuzzy->length_rejections = 0;
 
     OK;
@@ -850,6 +877,12 @@ FUNC (scan_file) (text_fuzzy_t * text_fuzzy, char * file_name,
     OK;
 }
 
+FUNC (alphabet_rejections) (text_fuzzy_t * text_fuzzy, int * r)
+{
+    * r = text_fuzzy->alphabet_rejections;
+    OK;
+}
+
 /* Free non-Perl malloc memory using the C library "free". This is all
    about "free to wrong pool". See
    "http://www.perlmonks.org/?node_id=742205" */
